
# Exceptions
from wordnik import RestfulError, InvalidRelationType, NoAPIKey, MissingParameters

# Globals
from wordnik import DEFAULT_URL, DEFAULT_FORMAT

from wordnik import Wordnik
Wordnik._populate_methods()

